; Auteur :	Laurent TANNOUS
; 			http://lolowebsite.free.fr
; Date : 12/03/2004 23h00
;
; Nom de la routine : onewire.asm
; Description : Gestion du protocole ONE WIRE de DALLAS
; Variables a dclarer (description):
; OWBUFFER		(variable servant au transfert sur le bus ONE WIRE)
; cmpt : 2		(compteur interne)
;
;
; Routines :
; OW_INIT : initialisation du bus ONE WIRE sur le PIC
; OW_RESET : Effectue un reset sur le bus, renvoie dans W : 'Y' ou 'N'
;			 selon que le reset ce soit bien pass ou non
; OW_READS : Effectue un "read slot", renvoie dans W : D'1' ou D'0'
; OW_WRITE1S : Effectue un "write 1 slot"
; OW_WRITE0S : Effectue un "write 0 slot"
; OW_WRITE8 : Ecrit l'octet contenu dans OWBUFFER sur le bus ONE WIRE
; OW_READ8 : Ecrit un octet lu sur le bus ONE WIRE dans la variable OWBUFFER
;
; Notes : 
;
; IMPORTANT : vitesse de fonctionnement : 4MHz (quartz ou oscillateur interne)
;
; Vous pouvez faire un copier-coller du contenu dans votre programme
; Ce programme utilise des macros de slection de banques que vous trouverez 
; dans la section "MACROS", elles sont mises en commentaire par dfaut, mais
; si vous n'en utilisez pas ou si vous n'utilisez pas les mmes,
; dcommentez - les !
; Ce programme propose, par ailleurs, une routine de temporisation rglable
; Que vous pourrez utiliser dans votre programme principal
; Il est  noter aussi que ce programme peut "consommer" trois niveaux dans
; la pile: on peut arriver  trois appels de sous-programmes.
;


; Definition du port et du bit sur lequel est install
; le ou les composants one wire

#define OW_BUS PORTA,4

; ****************************************************************************
;                               MACROS		         		     *
; ****************************************************************************

; La macro suivante sert  slectionner les banques mmoire, dcommentez-la
; si vous ne les avez pas dj mises dans votre programme principal

; ****************************************************************************
;BANK0	macro				; Selection bank 0
;		bcf	STATUS,RP0
;		bcf	STATUS,RP1
;	endm
;
;BANK1	macro				; Selection bank 1
;		bsf	STATUS,RP0
;		bcf	STATUS,RP1
;	endm
;
;BANK2	macro				; Selection bank 2
;		bcf	STATUS,RP0
;		bsf	STATUS,RP1
;	endm
;	
;BANK3	macro				; Selection bank 3
;		bsf	STATUS,RP1
;		bsf	STATUS,RP0
;	endm
; ****************************************************************************

; La macro suivante rgle le BUS en sortie et y impose un niveau bas
; Attention : les macros de slections de banques doivent tres prsents
OW_BUS_OUT Macro
	BANK1				; Banque 1
	bcf	OW_BUS			; = sortie
	BANK0				; Banque 0
	bcf	OW_BUS			; = niveau bas
	endM

; La macro suivante rgle le BUS en entre (on "relche" le bus)
OW_BUS_IN Macro
	BANK1				; Banque 1
	bsf	OW_BUS			; = entre
	BANK0				; Banque 0
	endM
; ****************************************************************************


; ****************************************************************************
; *		        Dbut des routines				     *
; ****************************************************************************


; ****************************************************************************
; Routine d'initialisation du bus ONE WIRE
OW_INIT
	; On doit tester qu'on a pas les rsistances de pull-up du PIC actifs !!!
	; 2 lignes pas utiles pour RA4
	;btfss		OPTION_REG, 7
	;bsf		OPTION_REG, 7
	OW_BUS_IN			; On rgle en entre le bus ONE WIRE
					; ( = haute impdance )
	return
; ****************************************************************************	


; ****************************************************************************
; Routine de reset du bus ONE WIRE
OW_RESET
; Premire tape : on force le bus  0 pendant au moins 480 s
	OW_BUS_OUT			; On rgle en sortie le bus ONE WIRE
    	call		t480		; Temporisation de 480 S
	OW_BUS_IN			; On relache le bus ONE WIRE	
	
; Deuxime partie : on attend une rponse sur le bus
; On va tester pendant 60 S le bus
; on a 5 s par itrations, donc 60/5=12 (on teste 12 fois le BUS)
	movlw	D'12'			; initialisation du compteur
	movwf	cmpt
sub_reset
	btfss	OW_BUS			; teste le bus :
	goto	Etape3			; si il n'est plus  1, c'est bon
	decfsz	cmpt			; si il est  1, on compte,
	goto	sub_reset		; puis on boucle
	retlw	'N'			; Si on a fini de compter sans succs, on renvoie
					; le caractre 'N' dans le registre W

; Troisime tape : on a eu un front descendant sur le bus, on doit valider
; alors un "presence pulse" en vrifiant qu'il dure au moins 60s
; on a 5 s par itrations, donc 60/5=12
Etape3
	movlw	D'12'			; initialisation du compteur
	movwf	cmpt
sub_etape3
	btfsc	OW_BUS			; teste le bus :
	retlw	'N'			; si il n'est plus  0, il y a un problme
	decfsz	cmpt			; si il est  0, on continue de compter
	goto	sub_etape3		; on boucle
					; Si on arrive  la fin du compteur : on a le temps
					; minimum, donc une "presence pulse" valide
	call	t480			; Temporisation de 480 S

	retlw	'Y'			; On termine en renvoyant 'Y'
; ****************************************************************************


; ****************************************************************************
; Routine READ SLOT
OW_READS
	; initialisation du compteur pour la temporisation rglable
	; (explications plus loin)
	movlw	D'20'
	movwf	cmpt
	; Etape 1 :
	; La squence suivant impose un niveau bas sur le bus pendant au moins 1s
	; valeur de scurit = 6s (moiti du temps max)
	OW_BUS_OUT			; impose un niveau bas
	NOP
	NOP
	NOP
	NOP
	; Attention : deux cycles pris par la macro OW_BUS_IN avant
	; et aprs le changement d'impdance
	OW_BUS_IN			; Relache le bus
	; Etape 2 :
	; On va lire maintenant sur le bus  la 15e s
	NOP	
	NOP
	NOP
	; l'exemple suivant teste deux fois le bus (pour tre sr)
	movlw	D'0'
	btfsc	OW_BUS			; teste le bus
	movlw	D'1'		
	btfsc	OW_BUS			; teste le bus
	movlw	D'1'		
	
	; Etape 3 : on attend jusqu' la fin du slot et on laisse un recovery time
	; On utilise la tempo rglable car on ne doit pas changer le contenu de W	
	call	temp

	return				; Fin de la routine
; ****************************************************************************


; ****************************************************************************
; Routine WRITE 1 SLOT

OW_WRITE1S
	; Etape 1
	; La squence suivante impose un niveau bas sur le bus pendant au moins 1s
	; valeur de scurit = 6s (moiti du temps max)
	OW_BUS_OUT			; sortie, niveau bas
	NOP
	NOP
	NOP
	NOP
	; Attention : deux cycles pris par la macro OW_BUS_IN avant
	; et aprs le changement d'impdance
	OW_BUS_IN			; Relache le bus
	
	; Etape 2 : on attend jusqu' la fin du slot et on laisse un recovery time
	call	t60			; tempo de 60 s
	
	return
; ****************************************************************************


; ****************************************************************************
; Routine Write 0 Slot ( attention : 0 = zro, pas la lettre 'O' )
	
OW_WRITE0S
	; Etape 1 : on impose un niveau bas sur le bus pendant toute la dure du slot
	OW_BUS_OUT
	call	t60
	OW_BUS_IN
	
	; Etape 2 : recovery time
	NOP
	NOP
	NOP
	NOP
	NOP

	
	return
; ****************************************************************************


; ****************************************************************************
; Routine Read 8 ( = lecture de 8 bits conscutifs )
; Rsultat dans OWBUFFER

OW_READ8
	movlw	D'8'			; initialisation du compteur
	movwf	cmpt+1
sub_READ8
	call	OW_READS		; lecture (READ SLOT), renvoie 1 ou 0 dans W
	xorlw	D'1'			; on teste la valeur dans W : le flag Z est positionn
	bcf		STATUS, C	; on prpositionne le flag C  0
	btfsc	STATUS, Z		; on teste le flag Z :
	bsf		STATUS, C	; si il est  1, on positionne le flag C
	rrf		OWBUFFER, F	; et on fait un dcallage dans le buffer avec le flag C

	decfsz	cmpt+1
	goto	sub_READ8 		; et on boucle

	return

; ****************************************************************************


; ****************************************************************************
; Routine Write 8 ( = criture de 8 bits conscutifs )
; octet  crire dans OWBUFFER

OW_WRITE8
	movlw	D'8'			; initialisation du compteur
	movwf	cmpt+1
sub_write8
	; la squence suivante teste un  un tous les bits de l'octet  envoyer
	; et choisit  chaque fois quel Write Slot utiliser

	btfsc	OWBUFFER, 0		; choix du Write slot
	call	OW_WRITE1S	
	btfss	OWBUFFER, 0		; choix du Write slot
	call	OW_WRITE0S
	rrf	OWBUFFER		; rotation pour tester le bit suivant

	decfsz	cmpt+1
	goto	sub_write8		; bouclage

	return
; ****************************************************************************


; ****************************************************************************
; Temporisations

; Temporisation de 480 s
; 3s par itrations, donc 160*3 = 480 s
; +2s pour l'initialisation (ngligeable)
t480
	movlw	D'160'
	movwf	cmpt
sub_t480
	decfsz	cmpt
	goto	sub_t480
	return

; Temporisation de 60 s
; 3s par itrations, donc 20*3 = 60 s
; +2s pour l'initialisation (ngligeable)
t60
	movlw	D'20'
	movwf	cmpt
sub_t60
	decfsz	cmpt
	goto	sub_t60
	return

; Temporisation rglabe : charger dans cmpt la valeur de la temporisation
; tempo = 3s*cmpt+2s

temp
	decfsz	cmpt
	goto	temp
	return
